@@ -4,7 +4,9 @@ SampleDB API package
 """
 
 from . import server
+from . import federation
 
 __all__ = [
-    'server'
+    'server',
+    'federation'
 ]
