@@ -14,7 +14,7 @@ SQLAlchemy ORM class.
 As the group models use flask-sqlalchemy however, the functions in this
 module should be called from within a Flask application context.
 """
-import collections
+import dataclasses
 import datetime
 import typing
 
@@ -22,7 +22,7 @@ import flask
 
 from .. import db
 from ..models import groups
-from .users import get_user
+from .users import check_user_exists, get_mutable_user
 from .security_tokens import generate_token
 from .notifications import create_notification_for_being_invited_to_a_group
 from ..logic.languages import get_language_by_lang_code, Language
@@ -33,28 +33,35 @@ from . import errors
 MAX_GROUP_NAME_LENGTH = 100
 
 
-class Group(collections.namedtuple('Group', ['id', 'name', 'description'])):
+@dataclasses.dataclass(frozen=True)
+class Group:
     """
     This class provides an immutable wrapper around models.groups.Group.
     """
-
-    def __new__(cls, id: int, name: dict, description: dict):
-        self = super(Group, cls).__new__(cls, id, name, description)
-        return self
+    id: int
+    name: typing.Dict[str, str]
+    description: typing.Dict[str, str]
 
     @classmethod
     def from_database(cls, group: groups.Group) -> 'Group':
-        return Group(id=group.id, name=group.name, description=group.description)
+        return Group(
+            id=group.id,
+            name=group.name,
+            description=group.description
+        )
 
 
-class GroupInvitation(collections.namedtuple('GroupInvitation', ['id', 'group_id', 'user_id', 'inviter_id', 'utc_datetime', 'accepted'])):
+@dataclasses.dataclass(frozen=True)
+class GroupInvitation:
     """
     This class provides an immutable wrapper around models.groups.GroupInvitation.
     """
-
-    def __new__(cls, id: int, group_id: int, user_id: int, inviter_id: int, utc_datetime: datetime.datetime, accepted: bool):
-        self = super(GroupInvitation, cls).__new__(cls, id, group_id, user_id, inviter_id, utc_datetime, accepted)
-        return self
+    id: int
+    group_id: int
+    user_id: int
+    inviter_id: int
+    utc_datetime: datetime.datetime
+    accepted: bool
 
     @classmethod
     def from_database(cls, group_invitation: groups.GroupInvitation) -> 'GroupInvitation':
@@ -68,12 +75,16 @@ class GroupInvitation(collections.namedtuple('GroupInvitation', ['id', 'group_id
         )
 
     @property
-    def expired(self):
+    def expired(self) -> bool:
         expiration_datetime = self.utc_datetime + datetime.timedelta(seconds=flask.current_app.config['INVITATION_TIME_LIMIT'])
-        return datetime.datetime.utcnow() >= expiration_datetime
+        return bool(datetime.datetime.utcnow() >= expiration_datetime)
 
 
-def create_group(name: typing.Union[str, dict], description: typing.Union[str, dict], initial_user_id: int) -> Group:
+def create_group(
+        name: typing.Union[str, typing.Dict[str, str]],
+        description: typing.Union[str, typing.Dict[str, str]],
+        initial_user_id: int
+) -> Group:
     """
     Creates a new group with the given names and descriptions and adds an
     initial user to it.
@@ -130,15 +141,20 @@ def create_group(name: typing.Union[str, dict], description: typing.Union[str, d
         if item[1] == '':
             del description[item[0]]
 
-    user = get_user(initial_user_id)
+    user = get_mutable_user(initial_user_id)
     group = groups.Group(name=name, description=description)
+    db.session.add(group)
     group.members.append(user)
     db.session.add(group)
     db.session.commit()
     return group
 
 
-def update_group(group_id: int, name: dict, description: dict) -> None:
+def update_group(
+        group_id: int,
+        name: typing.Dict[str, str],
+        description: typing.Dict[str, str]
+) -> None:
     """
     Updates the group's names and descriptions.
 
@@ -178,7 +194,7 @@ def update_group(group_id: int, name: dict, description: dict) -> None:
     if 'en' not in name:
         raise errors.MissingEnglishTranslationError()
 
-    group = groups.Group.query.get(group_id)
+    group = groups.Group.query.filter_by(id=group_id).first()
     if group is None:
         raise errors.GroupDoesNotExistError()
 
@@ -204,7 +220,7 @@ def delete_group(group_id: int) -> None:
     :raise errors.GroupDoesNotExistError: when no group with the given
         group ID exists
     """
-    group = groups.Group.query.get(group_id)
+    group = groups.Group.query.filter_by(id=group_id).first()
     if group is None:
         raise errors.GroupDoesNotExistError()
     # group permissions and group default permissions will be deleted due to
@@ -222,7 +238,7 @@ def get_group(group_id: int) -> Group:
     :raise errors.GroupDoesNotExistError: when no group with the given
         group ID exists
     """
-    group = groups.Group.query.get(group_id)
+    group = groups.Group.query.filter_by(id=group_id).first()
     if group is None:
         raise errors.GroupDoesNotExistError()
     return Group.from_database(group)
@@ -247,7 +263,7 @@ def get_group_member_ids(group_id: int) -> typing.List[int]:
     :raise errors.GroupDoesNotExistError: when no group with the given
         group ID exists
     """
-    group = groups.Group.query.get(group_id)
+    group = groups.Group.query.filter_by(id=group_id).first()
     if group is None:
         raise errors.GroupDoesNotExistError()
     return [user.id for user in group.members]
@@ -258,12 +274,12 @@ def get_user_groups(user_id: int) -> typing.List[Group]:
     Returns a list of the group IDs of all groups the user with the given
     user ID is a member of.
 
-    :param group_id: the ID of an existing user
+    :param user_id: the ID of an existing user
     :return: the member ID list
     :raise errors.UserDoesNotExistError: when no user with the given
         user ID exists
     """
-    user = get_user(user_id)
+    user = get_mutable_user(user_id)
     return [Group.from_database(group) for group in user.groups]
 
 
@@ -282,11 +298,11 @@ def invite_user_to_group(group_id: int, user_id: int, inviter_id: int) -> None:
         user ID already is a member of this group
     """
     # ensure the inviter exists
-    get_user(inviter_id)
-    group = groups.Group.query.get(group_id)
+    check_user_exists(inviter_id)
+    group = groups.Group.query.filter_by(id=group_id).first()
     if group is None:
         raise errors.GroupDoesNotExistError()
-    user = get_user(user_id)
+    user = get_mutable_user(user_id)
     if user in group.members:
         raise errors.UserAlreadyMemberOfGroupError()
 
@@ -326,10 +342,10 @@ def add_user_to_group(group_id: int, user_id: int) -> None:
     :raise errors.UserAlreadyMemberOfGroupError: when the user is already
         a member of the group
     """
-    group = groups.Group.query.get(group_id)
+    group = groups.Group.query.filter_by(id=group_id).first()
     if group is None:
         raise errors.GroupDoesNotExistError()
-    user = get_user(user_id)
+    user = get_mutable_user(user_id)
     if user in group.members:
         raise errors.UserAlreadyMemberOfGroupError()
     group.members.append(user)
@@ -340,9 +356,9 @@ def add_user_to_group(group_id: int, user_id: int) -> None:
         include_accepted_invitations=False
     )
     for invitation in invitations:
-        invitation = groups.GroupInvitation.query.filter_by(id=invitation.id).first()
-        invitation.accepted = True
-        db.session.add(invitation)
+        mutable_invitation = groups.GroupInvitation.query.filter_by(id=invitation.id).first()
+        mutable_invitation.accepted = True
+        db.session.add(mutable_invitation)
     db.session.commit()
 
 
@@ -363,10 +379,10 @@ def remove_user_from_group(group_id: int, user_id: int) -> None:
     :raise errors.UserNotMemberOfGroupError: when the user is not a member of
         the group
     """
-    group = groups.Group.query.get(group_id)
+    group = groups.Group.query.filter_by(id=group_id).first()
     if group is None:
         raise errors.GroupDoesNotExistError()
-    user = get_user(user_id)
+    user = get_mutable_user(user_id)
     if user not in group.members:
         raise errors.UserNotMemberOfGroupError()
     group.members.remove(user)
