@@ -4,6 +4,8 @@
 """
 
 import enum
+import typing
+
 import sqlalchemy.dialects.postgresql as postgresql
 from .. import db
 
@@ -16,7 +18,7 @@ class AuthenticationType(enum.Enum):
     API_TOKEN = 4
 
 
-class Authentication(db.Model):
+class Authentication(db.Model):  # type: ignore
     __tablename__ = "authentications"
 
     id = db.Column(db.Integer, primary_key=True)
@@ -26,17 +28,23 @@ class Authentication(db.Model):
     confirmed = db.Column(db.Boolean, default=False, nullable=False)
     user = db.relationship('User', backref="authentication_methods")
 
-    def __init__(self, login, authentication_type, confirmed, user_id):
+    def __init__(
+            self,
+            login: typing.Dict[str, typing.Any],
+            authentication_type: AuthenticationType,
+            confirmed: bool,
+            user_id: int
+    ) -> None:
         self.login = login
         self.type = authentication_type
         self.confirmed = confirmed
         self.user_id = user_id
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return '<{0}(id={1.id})>'.format(type(self).__name__, self)
 
 
-class TwoFactorAuthenticationMethod(db.Model):
+class TwoFactorAuthenticationMethod(db.Model):  # type: ignore
     __tablename__ = "two_factor_authentication_methods"
 
     id = db.Column(db.Integer, primary_key=True)
