@@ -4,6 +4,9 @@ Create default action types.
 """
 
 import os
+import typing
+
+import flask_sqlalchemy
 
 from ..actions import ActionType
 
@@ -11,20 +14,20 @@ MIGRATION_INDEX = 30
 MIGRATION_NAME, _ = os.path.splitext(os.path.basename(__file__))
 
 
-def run(db):
-    action_type_columns = db.session.execute("""
+def run(db: flask_sqlalchemy.SQLAlchemy) -> bool:
+    action_type_columns = db.session.execute(db.text("""
         SELECT column_name
         FROM information_schema.columns
         WHERE table_name = 'action_types';
-        """).fetchall()
+        """)).fetchall()
     pre_translation = ('name',) in action_type_columns
 
     existing_action_type_ids = [
         action_type[0]
-        for action_type in db.session.execute("""
+        for action_type in db.session.execute(db.text("""
                     SELECT id
                     FROM action_types;
-                """).fetchall()
+                """)).fetchall()
     ]
 
     # checks if translatable columns still exist in action_types
@@ -96,10 +99,10 @@ def run(db):
                 continue
 
             # Perform migration
-            db.session.execute("""
+            db.session.execute(db.text("""
                 INSERT INTO action_types (id, name, description, object_name, object_name_plural, view_text, perform_text, admin_only, show_on_frontpage, show_in_navbar, enable_labels, enable_files, enable_locations, enable_publications, enable_comments, enable_activity_log, enable_related_objects)
                 VALUES (:id, :name, :description, :object_name, :object_name_plural, :view_text, :perform_text, :admin_only, :show_on_frontpage, :show_in_navbar, :enable_labels, :enable_files, :enable_locations, :enable_publications, :enable_comments, :enable_activity_log, :enable_related_objects)
-            """, params=action_type)
+            """), params=action_type)
             performed_migration = True
 
         return performed_migration
@@ -149,26 +152,44 @@ def run(db):
             }
         ]
 
+        scicat_export_types = {
+            ActionType.SAMPLE_CREATION: 'SAMPLE',
+            ActionType.MEASUREMENT: 'RAW_DATASET',
+            ActionType.SIMULATION: 'RAW_DATASET'
+        }
+
         performed_migration = False
 
         existing_action_type_ids = [
             action_type[0]
-            for action_type in db.session.execute("""
+            for action_type in db.session.execute(db.text("""
                              SELECT id
                              FROM action_types
-                         """).fetchall()
+                         """)).fetchall()
         ]
 
         for action_type in default_action_types:
+            action_type_id: int = typing.cast(int, action_type['id'])
+
             # Skip migration by condition
-            if action_type['id'] in existing_action_type_ids:
+            if action_type_id in existing_action_type_ids:
                 continue
 
             # Perform migration
-            db.session.execute("""
+            db.session.execute(db.text("""
                           INSERT INTO action_types (id, admin_only, show_on_frontpage, show_in_navbar, enable_labels, enable_files, enable_locations, enable_publications, enable_comments, enable_activity_log, enable_related_objects)
                           VALUES (:id, :admin_only, :show_on_frontpage, :show_in_navbar, :enable_labels, :enable_files, :enable_locations, :enable_publications, :enable_comments, :enable_activity_log, :enable_related_objects)
-                      """, params=action_type)
+                      """), params=action_type)
             performed_migration = True
 
+            if action_type_id in scicat_export_types:
+                db.session.execute(db.text("""
+                    UPDATE action_types
+                    SET scicat_export_type = :export_type
+                    WHERE id = :id
+                """), {
+                    "id": action_type_id,
+                    "export_type": scicat_export_types[action_type_id]
+                })
+
         return performed_migration
