@@ -6,26 +6,27 @@ Usage: python -m sampledb create_instrument <name> <description>
 """
 
 import sys
+import typing
+
 from .. import create_app
 from ..logic.instruments import create_instrument, get_instruments
-from ..logic.instrument_translations import set_instrument_translation, get_instrument_with_translation_in_language
+from ..logic.instrument_translations import set_instrument_translation
 from ..logic.languages import Language
 
 
-def main(arguments):
+def main(arguments: typing.List[str]) -> None:
     if len(arguments) != 2:
         print(__doc__)
         exit(1)
     name, description = arguments
     app = create_app()
     with app.app_context():
+        if app.config['DISABLE_INSTRUMENTS']:
+            print('Error: instruments are disabled', file=sys.stderr)
+            exit(1)
         instruments = get_instruments()
         for instrument in instruments:
-            instrument = get_instrument_with_translation_in_language(
-                instrument_id=instrument.id,
-                language_id=Language.ENGLISH
-            )
-            if instrument.translation.name == name:
+            if instrument.name.get('en') == name:
                 print('Error: an instrument with this name already exists (#{})'.format(instrument.id), file=sys.stderr)
                 exit(1)
         instrument = create_instrument()
