@@ -6,18 +6,20 @@ Usage: python -m sampledb set_user_readonly <user_id> <yes_or_no>
 """
 
 import sys
+import typing
+
 from .. import create_app
 from ..logic.users import get_user, set_user_readonly
 from ..logic.errors import UserDoesNotExistError
 
 
-def main(arguments):
+def main(arguments: typing.List[str]) -> None:
     if len(arguments) != 2 or not all(arguments) or arguments[1] not in ('yes', 'no'):
         print(__doc__)
         exit(1)
-    user_id, yes_or_no = arguments
+    user_id_str, yes_or_no = arguments
     try:
-        user_id = int(user_id)
+        user_id = int(user_id_str)
     except ValueError:
         print("Error: user_id must be an integer", file=sys.stderr)
         exit(1)
